"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.availableParallelism = exports.getArchitectureSet = exports.getArchitectureName = exports.getArchitecture = exports.builtinModules = exports.openUrl = void 0;
const tslib_1 = require("tslib");
const fslib_1 = require("@yarnpkg/fslib");
const module_1 = tslib_1.__importDefault(require("module"));
const os_1 = tslib_1.__importDefault(require("os"));
const execUtils = tslib_1.__importStar(require("./execUtils"));
const miscUtils = tslib_1.__importStar(require("./miscUtils"));
const openUrlBinary = new Map([
    [`darwin`, `open`],
    [`linux`, `xdg-open`],
    [`win32`, `explorer.exe`],
]).get(process.platform);
exports.openUrl = typeof openUrlBinary !== `undefined`
    ? async (url) => {
        try {
            await execUtils.execvp(openUrlBinary, [url], { cwd: fslib_1.ppath.cwd() });
            return true;
        }
        catch {
            return false;
        }
    }
    : undefined;
function builtinModules() {
    // @ts-expect-error
    return new Set(module_1.default.builtinModules || Object.keys(process.binding(`natives`)));
}
exports.builtinModules = builtinModules;
const LDD_PATH = `/usr/bin/ldd`;
function getLibc() {
    var _a, _b, _c, _d;
    // It seems that Node randomly crashes with no output under some circumstances when running a getReport() on Windows.
    // Since Windows has no libc anyway, shortcut this path.
    if (process.platform === `win32`)
        return null;
    let header;
    try {
        header = fslib_1.xfs.readFileSync(LDD_PATH);
    }
    catch { }
    // Since the getReport can be prohibitely expensive (it also queries DNS which, if misconfigured, can take a long time to timeout),
    // we first check if the ldd binary is glibc or musl, and only then run the getReport() if we can't determine the libc variant.
    if (typeof header !== `undefined`) {
        if (header && header.includes(`GLIBC`))
            return `glibc`;
        if (header && header.includes(`musl`)) {
            return `musl`;
        }
    }
    const report = (_b = (_a = process.report) === null || _a === void 0 ? void 0 : _a.getReport()) !== null && _b !== void 0 ? _b : {};
    const sharedObjects = (_c = report.sharedObjects) !== null && _c !== void 0 ? _c : [];
    // Matches the first group if libc, second group if musl
    const libcRegExp = /\/(?:(ld-linux-|[^/]+-linux-gnu\/)|(libc.musl-|ld-musl-))/;
    return (_d = miscUtils.mapAndFind(sharedObjects, entry => {
        const match = entry.match(libcRegExp);
        if (!match)
            return miscUtils.mapAndFind.skip;
        if (match[1])
            return `glibc`;
        if (match[2])
            return `musl`;
        throw new Error(`Assertion failed: Expected the libc variant to have been detected`);
    })) !== null && _d !== void 0 ? _d : null;
}
let architecture;
let architectureSet;
function getArchitecture() {
    return architecture = architecture !== null && architecture !== void 0 ? architecture : {
        os: process.platform,
        cpu: process.arch,
        libc: getLibc(),
    };
}
exports.getArchitecture = getArchitecture;
function getArchitectureName(architecture = getArchitecture()) {
    if (architecture.libc) {
        return `${architecture.os}-${architecture.cpu}-${architecture.libc}`;
    }
    else {
        return `${architecture.os}-${architecture.cpu}`;
    }
}
exports.getArchitectureName = getArchitectureName;
function getArchitectureSet() {
    const architecture = getArchitecture();
    return architectureSet = architectureSet !== null && architectureSet !== void 0 ? architectureSet : {
        os: [architecture.os],
        cpu: [architecture.cpu],
        libc: architecture.libc ? [architecture.libc] : [],
    };
}
exports.getArchitectureSet = getArchitectureSet;
function availableParallelism() {
    // TODO: Use os.availableParallelism directly when dropping support for Node.js < 19.4.0
    if (`availableParallelism` in os_1.default)
        // @ts-expect-error - No types yet
        return os_1.default.availableParallelism();
    return Math.max(1, os_1.default.cpus().length);
}
exports.availableParallelism = availableParallelism;
